@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool 1.0.35  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.37  Copyright Tobias Oetiker, 1997 - 2000
  ****************************************************************************
  * rrd__graph.c  make creates ne rrds
  ****************************************************************************/
@@ -14,6 +14,14 @@
 #include <fcntl.h>
 #endif
 
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
+
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+    
 #define SmallFont gdLucidaNormal10
 #define LargeFont gdLucidaBold12
 
@@ -45,7 +53,7 @@ enum op_en {OP_NUMBER=0,OP_VARIABLE,OP_INF,OP_PREV,OP_NEGINF,
 	    OP_MIN,OP_MAX,OP_LIMIT, OP_FLOOR, OP_CEIL,
 	    OP_UN,OP_END};
 
-enum if_en {IF_GIF=0,IF_PNG=1};
+enum if_en {IF_GIF=0,IF_PNG=1,IF_GD=2};
 
 typedef struct rpnp_t {
     enum op_en   op;
@@ -185,6 +193,7 @@ typedef  struct graph_desc_t {
 #define ALTAUTOSCALE      0x02  /* use alternative algorithm to find lower and upper bounds */
 #define ALTAUTOSCALE_MAX  0x04  /* use alternative algorithm to find upper bounds */
 #define NOLEGEND          0x08  /* use no legend */
+#define ALTYMRTG          0x10  /* simulate mrtg's scaling */
 
 typedef struct image_desc_t {
 
@@ -214,7 +223,11 @@ typedef struct image_desc_t {
 				      probablility that the existing one is out of date */
     int            logarithmic;    /* scale the yaxis logarithmic */
     enum if_en     imgformat;         /* image format */
-    
+
+char* bkg_image; /* background image source */
+char* ovl_image; /* overlay image source */
+char* unit; /* measured value unit */
+
     /* status information */
     	    
     long           xorigin,yorigin;/* where is (0,0) of the graph */
@@ -257,11 +270,13 @@ int tzoffset(time_t);
 int print_calc(image_desc_t *, char ***);
 int leg_place(image_desc_t *);
 int horizontal_grid(gdImagePtr, image_desc_t *);
+int horizontal_mrtg_grid(gdImagePtr, image_desc_t *);
 int horizontal_log_grid(gdImagePtr, image_desc_t *);
 void vertical_grid( gdImagePtr, image_desc_t *);
 void axis_paint( image_desc_t *, gdImagePtr);
 void grid_paint( image_desc_t *, gdImagePtr);
 gdImagePtr MkLineBrush(image_desc_t *,long, enum gf_en);
+void copyImage(gdImagePtr gif, char *bkg_image, int copy_white);
 int lazy_check(image_desc_t *);
 int graph_paint(image_desc_t *, char ***);
 int gdes_alloc(image_desc_t *);
@@ -356,6 +371,7 @@ enum if_en if_conv(char *string){
     
     conv_if(GIF,IF_GIF)
     conv_if(PNG,IF_PNG)
+    conv_if(GD,IF_GD)
 
     return (-1);
 }
@@ -1053,6 +1069,16 @@ data_calc( image_desc_t *im){
 	    return -1;
 	}
 	
+        /* move the data pointers to the correct period */
+        for(rpi=0;im->gdes[gdi].rpnp[rpi].op != OP_END;rpi++){
+            if(im->gdes[gdi].rpnp[rpi].op == OP_VARIABLE){
+              long ptr = im->gdes[gdi].rpnp[rpi].ptr;
+              if(im->gdes[gdi].start > im->gdes[ptr].start) {
+                im->gdes[gdi].rpnp[rpi].data += im->gdes[gdi].rpnp[rpi].ds_cnt;
+              }
+            }
+        }
+        
 	/* step through the new cdef results array and calculate the values */
 	for (now = im->gdes[gdi].start;
 	     now<=im->gdes[gdi].end;
@@ -1967,6 +1993,7 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
     
     polyPoints[0].x=im->xorigin;
     polyPoints[1].x=im->xorigin+im->xsize;
+
     sgrid = (int)( im->minval / gridstep - 1);
     egrid = (int)( im->maxval / gridstep + 1);
     scaledstep = gridstep/im->magfact;
@@ -1974,7 +2001,7 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
 	polyPoints[0].y=ytr(im,gridstep*i);
 	if ( polyPoints[0].y >= im->yorigin-im->ysize
 	     && polyPoints[0].y <= im->yorigin) {
-	    if(i % labfact == 0){		
+	    if(i % labfact == 0){
 		if (i==0 || im->symbol == ' ') {
 		    if(scaledstep < 1){
 			if(im->extra_flags & ALTYGRID) {
@@ -2026,6 +2053,54 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
     return 1;
 }
 
+int
+horizontal_mrtg_grid(gdImagePtr gif, image_desc_t *im)
+{
+ int      i, xLeft, xRight, y;
+ double   scaledstep;
+ char     graph_label[100];
+ int      styleMajor[2];
+ int      decimals;
+ char     labfmt[64];
+
+ /* does the scale of this graph make it impossible to put lines
+    on it? If so, give up. */
+ if (isnan((im->maxval - im->minval) / im->magfact))
+  return 0;
+
+ styleMajor[0] = graph_col[GRC_MGRID].i;
+ styleMajor[1] = gdTransparent;
+
+ xLeft = im->xorigin;
+ xRight = im->xorigin + im->xsize;
+
+ decimals = ceil(log10(max(fabs(im->maxval),fabs(im->minval))));
+ scaledstep = ceil(25 * pow((double)10,-decimals) * im->maxval) * pow((double)10,decimals - 2) / im->magfact;
+
+ sprintf(labfmt,"%%4.%df",1 - (scaledstep > (double)10 || rint(scaledstep) == scaledstep));
+ if (im->symbol != ' ' || im->unit)
+  strcat(labfmt," ");
+ if (im->symbol != ' ')
+  sprintf(labfmt + strlen(labfmt),"%c",im->symbol);
+ if (im->unit)
+  strcat(labfmt,im->unit);
+
+ for (i = 0; i <= 4; i++)
+  {y = im->yorigin - im->ysize * i / 4;
+   if (y >= im->yorigin-im->ysize && y <= im->yorigin)
+    {sprintf(graph_label,labfmt,scaledstep * i);
+     gdImageString(gif,SmallFont,(xLeft - (strlen(graph_label) * SmallFont->w) - 7),
+                                 y - SmallFont->h / 2 + 1,
+                                 (unsigned char *)graph_label,graph_col[GRC_FONT].i);
+     gdImageSetStyle(gif,styleMajor,2);
+     gdImageLine(gif,xLeft  - 2,y,xLeft  + 2,y,graph_col[GRC_MGRID].i);
+     gdImageLine(gif,xRight - 2,y,xRight + 2,y,graph_col[GRC_MGRID].i);		    
+     gdImageLine(gif,xLeft,     y,xRight,    y,gdStyled);
+    }	
+  } 
+ return 1;
+}
+
 /* logaritmic horizontal grid */
 int
 horizontal_log_grid(gdImagePtr gif, image_desc_t   *im)
@@ -2292,6 +2367,10 @@ grid_paint(
       vertical_grid(gif, im);
     
     if (im->draw_y_grid == 1){
+	if (im->extra_flags & ALTYMRTG)
+	{res = horizontal_mrtg_grid(gif,im);
+	}
+	else
 	if(im->logarithmic){
 		res = horizontal_log_grid(gif,im);
 	} else {
@@ -2415,6 +2494,29 @@ MkLineBrush(image_desc_t *im,long cosel, enum gf_en typsel){
   }
   return brush;
 }
+
+void
+copyImage(gdImagePtr gif, char *image, int copy_white)
+{FILE *fi;
+ if ((fi = fopen(image,"rb")) != (FILE *)NULL)
+  {int i1, i2, c1, c2;
+   gdImagePtr img;
+   img = gdImageCreateFromGd(fi);		// load image
+   fclose(fi);
+   for (i1 = gdImageSX(img); --i1; )
+    for (i2 = gdImageSY(img); --i2; )
+     {c2 = gdImageGetPixel(img,i1,i2);	// get pixel
+      if (copy_white || img->red[c2] != 255 || img->green[c2] != 255 || img->blue[c2] != 255)
+       {if ((c1 = gdImageColorExact(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
+         if ((c1 = gdImageColorAllocate(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
+          c1 = gdImageColorClosest(gif,img->red[c2],img->green[c2],img->blue[c2]);
+        gdImageSetPixel(gif,i1,i2,c1);
+       }
+     }
+   gdImageDestroy(img);
+  }
+}
+
 /*****************************************************
  * lazy check make sure we rely need to create this graph
  *****************************************************/
@@ -2441,6 +2543,9 @@ int lazy_check(image_desc_t *im){
     case IF_PNG:
 	size = PngSize(fd,&(im->xgif),&(im->ygif));
 	break;
+    case IF_GD:
+	size = 0; // for now - GdSize(fd,&(im->xgif),&(im->ygif));
+	break;
     }
     fclose(fd);
     return size;
@@ -2547,10 +2652,13 @@ graph_paint(image_desc_t *im, char ***calcpr)
 				     im->gdes[i].col.green,
 				     im->gdes[i].col.blue);
     
-    
     /* the actual graph is created by going through the individual
        graph elements and then drawing them */
-    
+
+  if (im->bkg_image)	// background image added first
+   copyImage(gif,im->bkg_image,1);
+  else
+   {
     back[0].x = 0;
     back[0].y = 0;
     back[1].x = back[0].x+im->xgif;
@@ -2561,6 +2669,7 @@ graph_paint(image_desc_t *im, char ***calcpr)
     back[3].y = back[2].y;
 
     gdImageFilledPolygon(gif,back,4,graph_col[GRC_BACK].i);
+   }
 
     canvas[0].x = im->xorigin;
     canvas[0].y = im->yorigin;
@@ -2688,6 +2797,9 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	}
     }
 
+  if (im->ovl_image)	// overlay image added last
+   copyImage(gif,im->ovl_image,0);
+
     if (strcmp(im->graphfile,"-")==0) {
 #ifdef WIN32
         /* Change translation mode for stdout to BINARY */
@@ -2707,6 +2819,9 @@ graph_paint(image_desc_t *im, char ***calcpr)
     case IF_PNG:
 	gdImagePng(gif, fo);    
 	break;
+    case IF_GD:
+	gdImageGd(gif, fo);    
+	break;
     }
     if (strcmp(im->graphfile,"-") != 0)
 	fclose(fo);
@@ -2799,6 +2914,14 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 
     (*prdata)=NULL;
 
+#ifdef HAVE_TZSET
+    tzset();
+#endif
+#ifdef HAVE_SETLOCALE
+    setlocale(LC_TIME,"");
+#endif
+                                
+                                
     parsetime("end-24h", &start_tv);
     parsetime("now", &end_tv);
 
@@ -2828,6 +2951,10 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
     im.gdes = NULL;
     im.imgformat = IF_GIF; /* we default to GIF output */
 
+im.bkg_image = NULL; // do not put anything to background per default
+im.ovl_image = NULL; // do not use anything as overlay per default
+im.unit = 0; // do not display unit if not set
+
     for(i=0;i<DIM(graph_col);i++)
 	im.graph_col[i].red=-1;
     
@@ -2859,13 +2986,17 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 	    {"alt-autoscale-max", no_argument,    0,   259 },
 	    {"units-exponent",required_argument, 0,  260},
 	    {"step",       required_argument, 0,   261},
+	    {"alt-y-mrtg", no_argument,       0,   262 },
+	    {"background", required_argument, 0,  'B'},
+	    {"overlay",    required_argument, 0,  'O'},
+	    {"unit",       required_argument, 0,  'U'},
 	    {0,0,0,0}};
 	int option_index = 0;
 	int opt;
 
 	
 	opt = getopt_long(argc, argv, 
-			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:z:g",
+			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:z:gB:O:U:",
 			  long_options, &option_index);
 
 	if (opt == EOF)
@@ -2890,6 +3021,9 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 	case 261:
 	    im.step =  atoi(optarg);
 	    break;
+	case 262:
+	    im.extra_flags |= ALTYMRTG;
+	    break;
 	case 's':
 	    if ((parsetime_error = parsetime(optarg, &start_tv))) {
 	        rrd_set_error( "start time: %s", parsetime_error );
@@ -3036,6 +3170,15 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 	    strncpy(im.title,optarg,150);
 	    im.title[150]='\0';
 	    break;
+	case 'B':
+	    im.bkg_image = optarg;
+	    break;
+	case 'O':
+	    im.ovl_image = optarg;
+	    break;
+	case 'U':
+	    im.unit = optarg;
+	    break;
 
 	case '?':
             if (optopt != 0)
@@ -3356,4 +3499,3 @@ int bad_format(char *ptr) {
 	}
 	return 0;
 }
-
